PROGRAM Fisher_Distance_RealSpace
  USE cosmo
  USE growth
  USE linearpk
  USE nonlinearpk
  USE angular_distance
  ! A sample program for computing the error on the 1-dimensional distance
  ! scale, ln(R), from the galaxy power spectrum in real space, with 
  ! ln(amplitude), ns, and/or running index marginalized over.
  ! While no redshift space distortion is included in this code, 
  ! you may choose to include the Kaiser boost for the overall amplitude.
  ! Ref: Shoji, Jeong & Komatsu, arXiv:0805.4238v2 (ApJ in press)
  ! October 25, 2008: E.Komatsu
  IMPLICIT none
  integer :: npara=4                       ! # of parameters
  double precision, dimension(:),   allocatable :: der
  double precision, dimension(:,:), allocatable :: cov,fis,fistot
  integer,          dimension(:),   allocatable :: work,ind,imarg
  double precision :: kmax_ov_h,k_ov_h,k0_ov_h,kpivot=0.05/0.702 ! h Mpc^-1
  double precision :: linear_pk,dlnpkdlnk,nonlinear_pkd,dlnpkddlnk
  double precision :: neff,neffd,dlnk,factor,wkmu,dummy
  double precision :: pk,pkd,z,zin,bias,g,dgdlna,beta
  double precision :: zmin,zmax,area,Vsurvey,Vtot,ngal,number,Ntot
  character(len=128) :: filename
  integer :: n,i,j,ibin,nbins,ifile,inl,nmarg
  logical :: use_kaiser_boost
  external nonlinear_pkd,dlnpkddlnk,linear_pk,dlnpkdlnk,dgdlna,g
  ! Specify three cosmological parameters for computing the growth factor
  ! and the angular diameter distance. 
  ! The data type has been defined in MODULE cosmo.
  ode0=0.723d0
  om0=0.277d0
  w=-1d0
  CALL setup_growth ! tabulate the growth factor
  CALL setup_da     ! tabulate the angular diameter distance
  ! linear or non-linear?
  print*,'Use:'
  print*,'1) Linear power spectrum in real space'
  print*,'2) Non-linear power spectrum (3PT)  in real space'
  read*,inl
  print*,'Include the Kaiser boost for the amplitude?'
  print*,'1) Yes'
  print*,'0) No'
  read*,i
  if(i.eq.0)then
     use_kaiser_boost=.false.
  else
     use_kaiser_boost=.true.
  endif
  ALLOCATE(der(npara),fis(npara,npara),fistot(npara,npara))
  ! which parameters to marginalize over?
  print*,'How many parameters (out of 3: amplitude,ns,running)'
  print*,'   do you wish to marginalize over?'
  read*,nmarg
  if(nmarg>0)then
     ALLOCATE(imarg(nmarg))
     print*,'Select the parameters to marginalize over from:'
     print*,'1) ln(amplitude)'
     print*,'2) tilt (ns)'
     print*,'3) running index (alpha)'
     read*,imarg(1:nmarg)
  endif
  ! ask for survey specific parameters
  print*,'For the area, the redshift range and the number of galaxies:'
  print*,'1) Read in "number.txt"'
  print*,'2) Read in your favorite file'
  print*,'0) Enter your choice of area (in deg^2), zmin, zmax, # of galaxies in millions'
  print*,'   (A new file called "number.txt" will be created.)'
  read*,ifile
  if(ifile.eq.1)then
     filename='number.txt'
     open(2,file=filename,status='old')
  elseif(ifile.eq.2)then
     print*,'filename? (format: area in deg^2  zmin  zmax  # of galaxies in millions)'
     read*,filename
     open(2,file=filename,status='old')
  else
     filename='number.txt'
     open(2,file=filename,status='unknown')
  endif
  ! read in linear P(k)
  filename='wmap5baosn_max_likelihood_matterpower_at_z=30.dat'
  n=896    ! # of lines in the file
  zin=30d0 ! redshift of the input power spectrum
  CALL open_linearpk(filename,n)
  ! loop over redshift bins
  fistot=0d0 ! total Fisher matrix integrated over all the redshift bins
  Ntot=0d0   ! total number of galaxies in millions
  Vtot=0d0   ! total survey volume
  nbins=0    ! initialize the number of bins
  do ibin=1,100
     if(ifile.eq.1)then
        read(2,*,end=10)area,zmin,zmax,number
     elseif(ifile.eq.2)then
        read(2,*,end=10)area,zmin,zmax,number
     else
        print*,'Enter area (in deg^2), zmin, zmax, # of galaxies (in millions)'
        print*,'in the redshift bin#=(',ibin,')'
        print*,'(Enter 0,0,0,0 when you are done.)'
        read*,area,zmin,zmax,number
        if(area.eq.0d0.and.zmin.eq.0d0.and.zmax.eq.0d0.and.number.eq.0d0)goto 10
        write(2,'(4F18.5)')area,zmin,zmax,number
     endif
     nbins=nbins+1
     print*,'Enter the values of'
     print*,'bias and kmax [h Mpc^-1] in '
     print'(1F6.3,1A7,1F5.3)',zmin,'< z < ',zmax
     read*,bias,kmax_ov_h
     CALL volume(zmin,zmax,area,Vsurvey)
     z=0.5d0*(zmin+zmax)
     beta=(1d0+dgdlna(z)/g(z))/bias
     ngal=number*1d6/Vsurvey
     Ntot=Ntot+number
     Vtot=Vtot+Vsurvey
     ! computing the Fisher matrix...
     if(inl.ge.2)then
        ! read in non-linear P(k) density
        filename='wavenumber_pkd11_pkd22_pkd13_at_z=30.txt'
        n=508    ! # of lines in the file
        zin=30d0 ! redshift of the input power spectrum
        CALL open_nonlinearpkd(filename,n,g(z)/g(zin)*(1d0+zin)/(1d0+z))
        open(11,file=filename,status='old')
        read(11,*)k0_ov_h,dummy,dummy,dummy
     else
        open(11,file=filename,status='old')
        read(11,*)k0_ov_h,dummy
     endif
     k_ov_h=k0_ov_h
     fis=0d0
     do while (k_ov_h<=kmax_ov_h)
        if(inl.ge.2)then
           ! use non-linear P(k)
           read(11,*)k_ov_h,dummy,dummy,dummy
           pkd=nonlinear_pkd(k_ov_h)
           neffd=dlnpkddlnk(k_ov_h)
        else
           ! use linear P(k)
           read(11,*)k_ov_h,dummy
           pkd=linear_pk(k_ov_h)*(g(z)/g(zin)*(1d0+zin)/(1d0+z))**2d0
           neffd=dlnpkdlnk(k_ov_h)
        endif
        dlnk=dlog(k_ov_h)-dlog(k0_ov_h)
        factor=(k_ov_h)**3d0*dlnk/(2d0*3.1415926535d0**2d0) ! h^3 Mpc^-3
        if(use_kaiser_boost)then
           pk=bias**2d0*pkd*(1d0+2d0/3d0*beta+1d0/5d0*beta**2d0)
        else
           pk=bias**2d0*pkd
        endif
        neff=neffd
        wkmu=0.5d0*Vsurvey*(ngal*pk/(1d0+ngal*pk))**2d0
        der(1)=neff         ! dlnPkdlnR
        der(2)=1d0          ! dlnPkdln(amplitude)
        der(3)=dlog(k_ov_h/kpivot) ! dlnPkd(ns)
        der(4)=0.5d0*dlog(k_ov_h/kpivot)**2d0 ! dlnPkd(alpha)
        do i=1,npara
           fis(i,:)=fis(i,:)+factor*wkmu*der(i)*der(:)
        enddo
        k0_ov_h=k_ov_h
     enddo
     fistot=fistot+fis
     print*,'=== redshift bin#=(',ibin,') ==='
     print'(1F6.3,1A7,1F5.3)',zmin,'< z < ',zmax
     print'(1A7,1F8.5,1A11)','Vsur =',Vsurvey*1d-9,' h^-3 Gpc^3'
     print'(1A7,1F8.5,1A9)','Ngal =',number,' millions'
     print'(1A7,1F8.5,1A17)','ngal =',ngal*1d3,' 10^-3 h^3 Mpc^-3'
     print'(1A7,1F8.5)','bias =',bias
     print'(1A7,1F8.5)','beta =',beta
     print'(1A7,1F8.5,1A9)','kmax =',kmax_ov_h,' h Mpc^-1'
     CALL report_result(imarg,nmarg,npara,fis)
     print*,''
     if(inl.ge.2)then
        CALL close_nonlinearpkd
     endif
     close(11)
  enddo
10 close(2)
  CALL close_linearpk
  if(nbins>1)then
     print*,'=== combined ==='
     print'(1A7,1F8.5,1A11)','Vsur =',Vtot*1d-9,' h^-3 Gpc^3'
     print'(1A7,1F8.5,1A9)', 'Ngal =',Ntot,' millions'
     print'(1A7,1F8.5,1A17)','ngal =',Ntot*1d6/Vtot*1d3,' 10^-3 h^3 Mpc^-3'
     CALL report_result(imarg,nmarg,npara,fistot)
  endif
  DEALLOCATE(fis,fistot,der)
END PROGRAM Fisher_Distance_RealSpace
SUBROUTINE report_result(imarg,nmarg,npara,fis)
  IMPLICIT none
  integer, intent(IN) :: imarg(nmarg),nmarg,npara
  double precision, intent(IN) :: fis(npara,npara)
  double precision, allocatable, dimension(:,:) :: cov
  double precision, allocatable, dimension(:)   :: work
  double precision :: r12,err_lnda,err_lnh,err_lnR
  integer :: i,j
  ALLOCATE(cov(1+nmarg,1+nmarg),work(1+nmarg))
  if(nmarg>0)then
     cov(1,1)=fis(1,1)
     do i=1,nmarg
        cov(1,1+i)=fis(1,1+imarg(i))
        cov(1+i,1)=cov(1,1+i)
        do j=i,nmarg
           cov(1+i,1+j)=fis(1+imarg(i),1+imarg(j))
           cov(1+j,1+i)=cov(1+i,1+j)
        enddo
     enddo
     CALL DVERT(cov,1+nmarg,1+nmarg,work)
  else
     cov(1,1)=1d0/fis(1,1)
  endif
  err_lnR=sqrt(cov(1,1))
  print'(1A15,1F9.5)','Err[lnR](%)  =',err_lnR*1d2
  do i=1,nmarg
     if(imarg(i).eq.1)then
        print*,'*** marginalized over ln(amplitude) ***'
        print'(1A15,1F9.5)','Err[ln(A)][%]=',sqrt(cov(1+i,1+i))*1d2
     elseif(imarg(i).eq.2)then
        print*,'*** marginalized over tilt (ns) ***'
        print'(1A15,1F9.5)','Err[ns]      =',sqrt(cov(1+i,1+i))
     elseif(imarg(i).eq.3)then
        print*,'*** marginalized over running index (alpha) ***'
        print'(1A15,1F9.5)','Err[alpha]   =',sqrt(cov(1+i,1+i))   
     endif
  enddo
  DEALLOCATE(cov,work)
  return
END SUBROUTINE report_result
